﻿#undef ON_OFF
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using Arcelik.Okc.External;

namespace Arcelik
{
	public partial class Gui : Form
	{
		private GMP3Protocol controller = null;
		private int receiptID;
		private const int MAX_CONNECTION_DURATION = 90;
		private System.Timers.Timer timer;

		static public void Main ()
		{
			Application.Run (new Gui ());
		}

		public Gui()
		{
			InitializeComponent();

			button2.Click += new EventHandler (button2_Click);
			button4.Click += new System.EventHandler(button4_Click);
			button5.Click += new System.EventHandler(button5_Click);
			button6.Click += new System.EventHandler(button6_Click);
			button7.Click += new System.EventHandler(button7_Click);
			button8.Click += new System.EventHandler(button8_Click);
			button9.Click += new System.EventHandler(button9_Click);
			button10.Click += new System.EventHandler(button10_Click);

			textBox1.Click += new EventHandler (text1_Click);
			textBox2.Click += new EventHandler (text2_Click);
			textBox3.KeyPress += new KeyPressEventHandler (textBox3_KeyPress);

			MaximizeBox = false;

			textBox2.Text = isLinux() ? "/dev/ttyUSB0" : "com6";
		}

		private bool isLinux() {
			var p = (int) Environment.OSVersion.Platform;
			return (p == 4) || (p == 6) || (p == 128);
		}

		private void displayLibraryErrorMessage(ARStatus status)
		{
			#if false
			richTextBox1.Invalidate ();
			#endif
			richTextBox1.Select(0, 0);
			richTextBox1.SelectionColor = Color.Red;
			richTextBox1.SelectedText = status.ToString() + " => " + controller.strerror(status) + Environment.NewLine;
			richTextBox1.Refresh();
		}

		private void displayGeneralMessage(string message, Color color)
		{
			#if false
			richTextBox1.Invalidate ();
			#endif
			richTextBox1.Select(0, 0);
			richTextBox1.SelectionColor = color;
			richTextBox1.SelectedText = message + Environment.NewLine;
			richTextBox1.Refresh();
		}

		private ARStatus addDevice()
		{
			ARStatus rc;
			string terminalNo;
			terminalNo = textBox1.Text;
			if (terminalNo == String.Empty) {
				displayGeneralMessage ("AT Numarası Tanımlanmamış", Color.Red);
				return ARStatus.AR_ERROR_GENERIC;
			}
			rc = GMP3Protocol.addDevice (terminalNo);
			return rc;
		}

		private ARStatus addSerialPort()
		{
			ARStatus rc;
			string comPort;
			comPort = textBox2.Text;
			if (comPort == String.Empty) {
				displayGeneralMessage ("Seri Port Arayüzü Tanımlanmamış", Color.Red);
				return ARStatus.AR_ERROR_GENERIC;
			}

			rc = GMP3Protocol.addSerialPort (comPort);
			return rc;
		}

		private void enableConnectWidgets()
		{
			button1.Enabled = true;
			textBox1.Enabled = true;
			textBox2.Enabled = true;
		}

		private void button1_Click (object sender, EventArgs e)
		{	
			ARStatus rc;
			string terminalNo;

			button1.Enabled = false;
			textBox1.Enabled = false;
			textBox2.Enabled = false;
			textBox1.ForeColor = System.Drawing.SystemColors.ActiveBorder;
			textBox2.ForeColor = System.Drawing.SystemColors.ActiveBorder;

			terminalNo = textBox1.Text;

			controller = new GMP3Protocol (terminalNo);

			rc = addDevice ();
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				enableConnectWidgets ();
				return;
			}

			rc = addSerialPort ();
			if (rc != ARStatus.Success && rc != ARStatus.AR_ERROR_SERIAL_INTERFACE_ALREADY_ADDED) {
				displayLibraryErrorMessage (rc);
				enableConnectWidgets ();
				return;
			}

			rc = GMP3Protocol.startService ();
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				enableConnectWidgets ();
				return;
			}

			displayNETlibVersion ();

			pair2Device ();
		}

		private void pair2Device() {
			int count = 0;
			timer = new System.Timers.Timer();
			timer.Elapsed += new ElapsedEventHandler((sender, e) => {
				displayGeneralMessage("Cihazla eşleşme sağlanıyor..", Color.Black);
				++count;
				if (controller.isDevicePaired()) {
					timer.Enabled = false;
					displayGeneralMessage("Eşleşme sağlandı", Color.Blue);
					displayOkcVersion();
				}
				if (count >= MAX_CONNECTION_DURATION) {
					timer.Enabled = false;
					displayGeneralMessage("Cihazla Eşleşme sağlanamadı", Color.Red);
					button1.Enabled = true;
					textBox1.Enabled = true;
					textBox2.Enabled = true;
				}
			});

			timer.Interval = 1000;
			timer.Enabled = true;
		}

		private void displayOkcVersion()
		{
			string version = "version";
			controller.getOkcSoftwareVersion (ref version);
			label10.Text = version;
		}

		private void displayNETlibVersion()
		{
			string version = "";
			controller.getNETLibVersion (ref version);
			label8.Text = version;
		}

		private void displaySubtotal() 
		{
			ARStatus rc;
			double restAmount = 0;
			rc = controller.receiptSubtotal(receiptID, ref restAmount, SubtotalType.OnlyRead);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				return;
			}
			label6.Text = restAmount.ToString () + " TL";
			displayGeneralMessage("Kalan ödeme tutarı: " + restAmount.ToString (), Color.BlueViolet);
			textBox3.Text = restAmount.ToString ();
		}

		private void button2_Click (object sender, EventArgs e)
		{
			richTextBox1.Text = "";
		}

		private void button3_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			ARStatus rc;
			double amount = 2.5;
			Item item = new Item(button3.Text, 1, amount, SectionID.SID1);
			rc = controller.printProductItem(item);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				return;
			}
			displayGeneralMessage(button3.Text + ": " + amount.ToString(), Color.Black);
			displaySubtotal ();
		}

		private void button4_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			ARStatus rc;
			double amount = 6.5;
			Item item = new Item(button4.Text, 1, amount, SectionID.SID1);
			rc = controller.printProductItem(item);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				return;
			}
			displayGeneralMessage(button4.Text +  ": " + amount.ToString(), Color.Black);
			displaySubtotal ();
		}

		private void button5_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			ARStatus rc;
			double amount = 4.5;
			Item item = new Item(button5.Text, 1, amount, SectionID.SID1);
			rc = controller.printProductItem(item);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				return;
			}
			displayGeneralMessage(button5.Text +  ": " + amount.ToString(), Color.Black);
			displaySubtotal ();
		}

		private void button7_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			ARStatus rc;
			double amount;

			try {
				if (textBox3.Text == String.Empty) amount = 0;
				else amount = Convert.ToDouble(textBox3.Text);
			}   
			catch (FormatException) {
				displayGeneralMessage("Tutar formatı uygun değil. Yeni bir tutar giriniz.", Color.Red);
				return;
			}               
			catch (OverflowException) {
				displayGeneralMessage("Tutarı kontrol ederek yeniden giriniz.", Color.Red);
				return;
			}

			rc = controller.paymentItem("NAKİT", PaymentType.CASH, CurrencyID.TL, receiptID, amount, BankOperatorId.SHOW_BANK_OPERATOR_LIST);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				Console.WriteLine("Error: {0} => {1}", rc, controller.strerror(rc));

				return;
			}
			displayGeneralMessage("Ödeme Başarılı", Color.Blue);

			displaySubtotal ();
		}

		private void button6_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			ARStatus rc;
			double amount;

			try {
				if (textBox3.Text == String.Empty) amount = 0;
				else amount = Convert.ToDouble(textBox3.Text);
			}   
			catch (FormatException) {
				displayGeneralMessage("Tutar formatı uygun değil. Yeni bir tutar giriniz.", Color.Red);
				return;
			}               
			catch (OverflowException) {
				displayGeneralMessage("Tutarı kontrol ederek yeniden giriniz.", Color.Red);
				return;
			}

			rc = controller.paymentItem("KUPON", PaymentType.TICKET, CurrencyID.TL, receiptID, amount, BankOperatorId.SHOW_BANK_OPERATOR_LIST);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				Console.WriteLine("Error: {0} => {1}", rc, controller.strerror(rc));

				return;
			}
			displayGeneralMessage("Ödeme Başarılı", Color.Blue);
			displaySubtotal ();
		}

		private void button10_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			ARStatus rc;
			double amount;

			try {
				if (textBox3.Text == String.Empty) amount = 0;
				else amount = Convert.ToDouble(textBox3.Text);
			}   
			catch (FormatException) {
				displayGeneralMessage("Tutar formatı uygun değil. Yeni bir tutar giriniz.", Color.Red);
				return;
			}               
			catch (OverflowException) {
				displayGeneralMessage("Tutarı kontrol ederek yeniden giriniz.", Color.Red);
				return;
			}

			rc = controller.paymentItemFoodCard("YEMEK KARTI", CurrencyID.TL, receiptID, amount, FoodCardOperatorId.SHOW_FOOD_OPERATOR_LIST);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
				Console.WriteLine("Error: {0} => {1}", rc, controller.strerror(rc));

				return;
			}
			displayGeneralMessage("Ödeme Başarılı", Color.Blue);
			displaySubtotal ();
		}
			
		private void button8_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			ARStatus rc;
			double amount;
#if ON_OFF
			int flag = 0;
#endif

			GMP3Protocol.OnPaymentResultCallback cb;

			cb = (ARStatus result) => {
#if ON_OFF
				/*callback 1 kez çağrılmalı*/
				if (Interlocked.CompareExchange(ref flag, 1, 0) != 0) {
					return;
				}
#endif
				if (result == ARStatus.Success) {
					Console.WriteLine("###Ödeme Başarılı###");
					displayGeneralMessage("Ödeme işlemi başarılı", Color.Blue);
					/*Ödeme cihaz cradle üzerinde değilken yapılmışsa cihaz yerine geri konulduğunda callback
					çağrılmasına karşın cihaza gönderilen mesajlar kısa bir süre AR_ERROR_MESSAGE_NOT_SENT hatası dönüyor.
					Bu yüzden 100 ms bekleniyor*/
#if true
					Thread.Sleep(1000);
#endif
					displaySubtotal();
				} else {
					displayGeneralMessage("Ödeme işlemi başarısız", Color.Red);
					displayLibraryErrorMessage (result);
				}
			};

			try {
				if (textBox3.Text == String.Empty) amount = 0;
				else amount = Convert.ToDouble(textBox3.Text);
			}   
			catch (FormatException) {
				displayGeneralMessage("Tutar formatı uygun değil. Yeni bir tutar giriniz.", Color.Red);
				return;
			}               
			catch (OverflowException) {
				displayGeneralMessage("Tutarı kontrol ederek yeniden giriniz.", Color.Red);
				return;
			}

#if OFF_ON
			double rest_amount_begin = -1;
			rc = controller.receiptSubtotal(receiptID, ref rest_amount_begin, SubtotalType.OnlyRead);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
			}
#endif
			rc = controller.paymentItemCredit("KREDİ", CurrencyID.TL, receiptID, cb, amount, BankOperatorId.SHOW_BANK_OPERATOR_LIST);
			if (rc != ARStatus.Success) {
				displayLibraryErrorMessage (rc);
			}
			else {
				displayGeneralMessage("Ödeme isteği başarılı şekilde gönderildi", Color.Blue);
			}
#if OFF_ON
			do {
				Thread.Sleep(1000);
				bool status = controller.isDevicePaired();
				if (status == false) {
					continue;
				}

				ReceiptStatus rstatus = new ReceiptStatus();
				rc = controller.queryReceiptStatus(receiptID, ref rstatus);
				/*banka uygulaması açıkken ÖKC bir süre meşgul moda geçiyor*/
				if (rc == ARStatus.AR_ERROR_DEVICE_NOT_RESPONDING) {
					continue;
				}

				if (rstatus.statusCode == 3 || rstatus.statusCode == 4) {
					double rest_amount_end = -1;
					controller.receiptSubtotal(receiptID, ref rest_amount_end, SubtotalType.OnlyRead);

					if (rest_amount_begin - rest_amount_end > 0) {
						/*Ödeme işlemi başarılı*/
						cb(ARStatus.Success);
					} else {
						/*Ödeme işlemi başarısız, genel hata dönüyoruz*/
						cb(ARStatus.AR_ERROR_GENERIC);
					}
					break;
				}
				if (rstatus.statusCode == 5) {
					/*Ödeme işlemi başarılı. Fiş tamamlandı*/
					cb(ARStatus.Success);
					break;
				}
				if (rstatus.statusCode == 6) {
					Console.WriteLine("Fiş iptal edildi");
#if true
					cb(ARStatus.AR_ERROR_GENERIC);
#endif
					break;
				}
			} while(true);
#endif
		}

		private void button9_Click (object sender, EventArgs e)
		{
			if (controller == null) {
				displayGeneralMessage("Cihaz Bağlantısı Gerçekleştirilmemiş", Color.Red);
				return;
			}

			receiptID = controller.beginSaleReceipt ();

			if (receiptID == (int)ARStatus.AR_ERROR_OKC_SALE_SCREEN) {
				displayGeneralMessage("Cihaz Satış Ekranında Değil", Color.Black);
				return;
			}

			if (receiptID < 0) {
				displayGeneralMessage("Satış işlemine başlanamadı", Color.Red);
				displayLibraryErrorMessage((ARStatus)receiptID);
				return;
			}

			if (receiptID == 2) {
				displayGeneralMessage("Satış işlemine başlanamadı", Color.Red);
				return;
			}

			displayGeneralMessage("Receipt id: " + receiptID.ToString(), Color.Blue);
			displayGeneralMessage ("Ürün veya ürünler girilebilir", Color.Blue);
		}

		private void text1_Click (object sender, EventArgs e)
		{
			this.textBox1.ForeColor = System.Drawing.SystemColors.ControlText;
		}

		private void text2_Click (object sender, EventArgs e)
		{
			this.textBox2.ForeColor = System.Drawing.SystemColors.ControlText;
		}

		private void label1_Click (object sender, EventArgs e)
		{
		}

		private void textBox1_TextChanged (object sender, EventArgs e)
		{
		}

		private void textBox3_KeyPress (object sender, KeyPressEventArgs e)
		{
			if (!char.IsControl(e.KeyChar) && !char.IsDigit(e.KeyChar) && (e.KeyChar != '.')) {
				e.Handled = true;
			}

			if ((e.KeyChar == '.') && ((sender as TextBox).Text.IndexOf('.') > -1)) {
				e.Handled = true;
			}
		}

		private void Okc_Load (object sender, EventArgs e)
		{
		}
	}
}
